/*
 * Locus.java
 *
 * Created on February 28, 2005, 6:42 PM
/* Copyright  2005-2011 Stephan Pelikan and Steven Rogstad
/* contact:  pelikan@math.uc.edu
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU Library General Public License as published 
/* by  the Free Software Foundation; either version 3 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU Library General Public License for more details.
/*
/* You should have received a copy of the GNU  General Public License
/* along with this program; see the file gpl-3.0.txt  If not, write to 
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
/* Boston, MA  02111-1307 USA
 */

/**
 * represents and stores all the data we need about a locus
 * @author sep
 */
import java.util.*;

public class Locus {

    DiscreteProbabilityDistribution dpd; //probs of the alleles
    DPD genotypeDPD; //probs of genotypes if F >0
    float F = 0.0f;
    private static final boolean Debug = false;

    /** Creates a new instance of Locus */
    public Locus() {
        dpd = new DiscreteProbabilityDistribution();
        genotypeDPD = null;
    }

    /**
     * 
     * @param ibc the value of F "inbreeding coefficient" for this locus
     *   to be used in generating random plants
     */
    public void setF(float ibc) {
        F = ibc;
        if (F > 0.0f) {
            genotypeDPD = new DPD();
            //we store the genotypes as Lists of alleles
            for (int a = 0; a < dpd.number_points(); a++) {
                for (int b = a; b < dpd.number_points(); b++) {
                    ArrayList genotype = new ArrayList();
                    genotype.add(new Integer(a));
                    genotype.add(new Integer(b));
                    float p, pa, pb;
                    pa = dpd.p[a];
                    pb = dpd.p[b];
                    if (a == b) {
                        p = (1 - F) * pa * pa + F * pa;
                    } else {
                        p = 2 * (1 - F) * pa * pb;
                    }
                    genotypeDPD.addPoint(p, genotype);
                }

            }
            genotypeDPD.normalize();
        } else {
            genotypeDPD = null;
        }
    }

    public Locus(float ibc) {
        dpd = new DiscreteProbabilityDistribution();
        F = ibc;
        if (F > 0.0f) {
            genotypeDPD = new DPD();
            //we store the genotypes as Lists of alleles
            for (int a = 0; a < dpd.number_points(); a++) {
                for (int b = a; b < dpd.number_points(); b++) {
                    ArrayList genotype = new ArrayList();
                    genotype.add(new Integer(a));
                    genotype.add(new Integer(b));
                    float p, pa, pb;
                    pa = dpd.p[a];
                    pb = dpd.p[b];
                    if (a == b) {
                        p = (1 - F) * pa * pa + F * pa;
                    } else {
                        p = 2 * (1 - F) * pa * pb;
                    }
                    genotypeDPD.addPoint(p, genotype);
                }
                genotypeDPD.normalize();
            }
        } else {
            genotypeDPD = null;
        }
    }

    /**
     * the number of alleles at this locus
     * @return number of alleles at this locus
     */
    public int number_alleles() {
        return dpd.number_points();
    }

    /**
     *       pickAllele(double r) uses the uniform 0-1 random number r
     *       to select one of the alleles from this locus according to the
     *       frequency distribution []p.
     * @param r a random number to be used in selecting one of the alleles at this locus
     * @return the allele selected
     */
    public int pickAllele(double r) {
        return (int) Math.round((double) dpd.pickOne(r));
    }

    /**
     * 
     * @param rnum
     * @return a list of the indices for the two alleles at this locus
     */
    public int[] pickGenotype(Random rnum) {
        int[] ans = new int[2];
        if (F == 0.0f) {
            // if(Debug) System.out.println("You probably shouldn't be calling this function with F=0");
            for (int i = 0; i < 2; i++) {
                ans[i] = (int) Math.round((double) dpd.pickOne(rnum.nextDouble()));
            }
        } else {
            ArrayList gt = (ArrayList) genotypeDPD.pickOne(rnum.nextDouble());

            for (int i = 0; i < 2; i++) {
                ans[i] = ((Integer) gt.get(i)).intValue();
            }

        }
        return ans;
    }

    /**
     * @return a string giving XML markup for this locus
     */
    public String toXML() {
        StringBuilder sb = new StringBuilder("");
        sb.append("<locus>\n");
        sb.append(dpd.toXML());
        sb.append("</locus>\n");
        return sb.toString();
    }
}
