
/*
 * Region.java
 */* Copyright  2005-2011 Stephan Pelikan and Steven Rogstad
/* contact:  pelikan@math.uc.edu
/*
/* This program is free software; you can redistribute it and/or modify
/* it under the terms of the GNU Library General Public License as published 
/* by  the Free Software Foundation; either version 3 of the License or
/* (at your option) any later version.
/*
/* This program is distributed in the hope that it will be useful, but
/* WITHOUT ANY WARRANTY; without even the implied warranty of
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/* GNU Library General Public License for more details.
/*
/* You should have received a copy of the GNU  General Public License
/* along with this program; see the file gpl-3.0.txt  If not, write to 
/* the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
/* Boston, MA  02111-1307 USA

 * Created on July 15, 2005, 3:57 PM
Modified to work with continuous locations instead of grid points
22 July 2006

We now simply implement a single containing rectangle;
Later we'll allow more complicated regions tha are unions of
rectangles with sides vertical and horizontal (at least)
 * 
 * a Region is contained inside a big, bounding rectangle.
 * If no subregions (rectangles or convex polygons) are defined,
 * a point belongs to the region if it belongs to the bounding rectangle.
 * 
 * Otherwise, the point must also belong to one of the subregions.
 * 
 * (Union (subregions)) INTERSECT (bounding rectangle)
 */
/**
 *
 * @author sep
 */
import java.util.*;
import java.text.NumberFormat;

public class Region extends Rect {

    List<Rect> rectangles;
    List<ConvexPolygon> Polygons;

    public Region(float xl, float yl, float xh, float yh) {
        super(xl, yl, xh, yh);
        rectangles = new ArrayList<Rect>();
        Polygons = new ArrayList<ConvexPolygon>();
    }

    public void addRect(Rect r) {
        rectangles.add(r);
    }

    public void addPoly(ConvexPolygon p) {
        Polygons.add(p);
    }

    public boolean inRegion(float x, float y) {
//false if outside the big bounding rectangle
        if (!this.inRect(x, y)) {
            return false;
        }
//otherwise, if no subregions are defined, true	
        if (rectangles.size() == 0 && Polygons.size() == 0) {
            return true;
        }
//and if there's subregions defined, true only if we're in one of 'em.
        Iterator it = rectangles.iterator();
        while (it.hasNext()) {
            Rect r = (Rect) it.next();
            if (r.inRect(x, y)) {
                return true;
            }
        }
        //if we have convexpolygons
        it = Polygons.iterator();
        while (it.hasNext()) {
            ConvexPolygon p = (ConvexPolygon) it.next();
            if (p.isInside(x, y)) {
                return true;
            }
        }

        return false;

    }

    @Override
    public String toXML() {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(2);
        StringBuilder sb = new StringBuilder("");
        sb.append("<Region XL=\"" + nf.format(XL) + "\" ");
        sb.append("YL=\"" + nf.format(YL) + "\" ");
        sb.append("XH=\"" + nf.format(XH) + "\" ");
        sb.append("YH=\"" + nf.format(YH) + "\" >\n");
        Iterator it = rectangles.iterator();
        while (it.hasNext()) {
            Rect r = (Rect) it.next();
            sb.append(r.toXML());
        }

        it = Polygons.iterator();
        while (it.hasNext()) {
            ConvexPolygon p = (ConvexPolygon) it.next();
            sb.append(p.toXML());
        }
        sb.append("\n</Region>\n");
        return sb.toString();
    }

    public static void main(String[] args) {
        Region r = new Region(-2, -2, 2, 2);
        Rect r1 = new Rect(0, 0, 1, 1);
        r.addRect(r1);
        System.out.println(r.inRegion(-3, 3));
        System.out.println(r.inRegion(0, 1));
        System.out.println(r.toXML());
    }
}
